/*******************************************************************************
*                                                                              *
*                    Progetto di Grafica in Xcmodel                            *
*                            "Red Baron"                                       *
*                           Flavio Bertini                                     *
*                                                                              *
*             redbaron.c: contiene la descrizione della scena,                 *
*            per la resa realistica tramite il pacchetto xcrayt.               *
*                                                                              *
*******************************************************************************/

#include <math.h>
#include "descriptor/descriptor.h"

//#define ANIMAZIONI

#define R 0
#define G 1
#define B 2

int main(void) {

  int anim;
  float step_fenice[8][3];
  float step_apparizione[10];
  float step_avanzamento = 0;

  /* Step della luce da cui appare il Barone */
  step_fenice[0][R] = 1;  step_fenice[0][G] = 0.921961;  step_fenice[0][B] = 0.470099;
  step_fenice[1][R] = 1;  step_fenice[1][G] = 0.811765;  step_fenice[1][B] = 0.402942;
  step_fenice[2][R] = 1;  step_fenice[2][G] = 0.7;       step_fenice[2][B] = 0.335785;
  step_fenice[3][R] = 0.99;  step_fenice[3][G] = 0.588235;  step_fenice[3][B] = 0.268628;
  step_fenice[4][R] = 0.98;  step_fenice[4][G] = 0.441176;  step_fenice[4][B] = 0.201471;
  step_fenice[5][R] = 0.96;  step_fenice[5][G] = 0.294118;  step_fenice[5][B] = 0.134314;
  step_fenice[6][R] = 0.93;  step_fenice[6][G] = 0.147059;  step_fenice[6][B] = 0.067157;
  step_fenice[7][R] = 0.9;   step_fenice[7][G] = 0.0;       step_fenice[7][B] = 0.0;

  /* Delta Z per la traslazione del Barone in fase di apparizione */
  step_apparizione[0] = -0.88998; step_apparizione[1] = -0.88911;
  step_apparizione[2] = -0.88737;  step_apparizione[3] = -0.88389;
  step_apparizione[4] = -0.87693;  step_apparizione[5] = -0.86302;
  step_apparizione[6] = -0.83517;  step_apparizione[7] = -0.77949;
  step_apparizione[8] = -0.66814;  step_apparizione[9] = -0.44543;

  /*** Scena ***/
  ITEM scena, fokker;

  /*** Elementi della Scena: pareti, arredo, oggetti ***/
  ITEM pareti, soffitto, pavimento, tavolo_piano, tavolo_base,
       boccale, boccale_manico, fgl1, fgl2, qdr1, qdr2, qdr1_c, qdr2_c,
       lampadario, lamp_aggancio, lamp_asta, lamp_lampadina,
       sedia_seduta, sedia, mensola, specchio_superficie,specchio_cornice,
       lbr1_copertina, lbr1_costa, lbr1_pagine,
       lbr2_copertina, lbr2_costa, lbr2_pagine, lbr2_retro,
       lbr3_copertina, lbr3_costa, lbr3_pagine, lbr3_retro;

  /*** Elementi del Fokker ***/
  ITEM parti_rosse, parti_nere ,parti_bianche, parti_legno, parti_metalliche,
       elica, ala_I, fusoliera_posteriore, timone, bordo_pilota;

  /*** Attributi ***/
  ITEM intonaco, intonaco_chiaro, carta, legno, ceramica, vetro, marrone_scuro,
       marrone_chiaro, specchio, rame, metallo, plastica_bianca, plastica_nera,
       plastica_rossa, plastica_marrone;

  /*** Attributi ***/
  ITEM lampadina, luce1, luce2, fenice;

  /* Definizione Luci */
  set_background(0.676,0.861,0.953,TRUE);
  set_ambient_light(1,1,1,0.7);
  lampadina = create_warn_light(0.0,0.0,22.44, 0.0,0.0,-1.0, 1,1,0.7, 5.5,35,0.3,45, 0,0,0,0,0, "lampadina");
  luce1 = create_point_light(5.0,-30.0,26.0, 1,1,0.9, 1.2, 100, "luce1");
  luce2 = create_point_light(-33.0,-30.0,19.5, 1,1,0.9, 1.5, 180, "luce2");

  /* Attributo Intonaco */
  intonaco = create_attribute("intonaco");
  set_color(intonaco, 0.91,0.9,0.89, 0.12,0.7);
  set_reflectivity(intonaco, 0, 12);

  /* Attributo Intonaco Chiaro */
  intonaco_chiaro = create_attribute("intonaco_chiaro");
  set_color(intonaco_chiaro, 0.92,0.92,0.92, 0.2,0.8);
  set_reflectivity(intonaco_chiaro, 0.3, 12);

  /* Attributo Carta */
  carta = create_attribute("carta");
  set_color(carta, 0.63,0.64,0.60, 0.05,0.22);
  set_reflectivity(carta, 0.06, 1);

  /* Attributo Legno */
  legno = create_attribute("legno");
  set_color(legno, 0.6,0.4,0.3, 0.1,0.8);
  set_reflectivity(legno, 0.1, 100);

  /* Attributo Ceramica */
  ceramica = create_attribute("ceramica");
  set_color(ceramica, 0.89,0.81,0.71, 0.1,0.8);
  set_reflectivity(ceramica, 0.22, 128);

  /* Attributo Vetro */
  vetro = create_attribute("vetro");
  set_color(vetro, 0.8078,0.9089,0.9019, 0.6,0.6);
  set_reflectivity(vetro, 0.26, 1);
  set_transparency(vetro, 1, 0.99, 2, 1.55);

  /* Attributi Marrone Chiaro e Scuro */
  marrone_chiaro = create_attribute("marrone_chiaro");
  set_color(marrone_chiaro, 0.914,0.522,0.231, 0.1,0.8);
  set_reflectivity(marrone_chiaro, 0, 12);
  marrone_scuro = create_attribute("marrone_scuro");
  set_color(marrone_scuro, 0.643,0.337,0.114, 0.1,0.8);
  set_reflectivity(marrone_scuro, 0, 12);

  /* Attributo Specchio */
  specchio = create_attribute("specchio");
  set_color(specchio, 1,1,1, 0,0);
  set_reflectivity(specchio, 0.8, 10);

  /* Attributo Rame */
  rame = create_attribute("rame");
  set_color(rame, 0.902,0.522,0.209, 0.4,0.5);
  set_reflectivity(rame, 0.8, 40);
  set_transparency(rame, 0.1, 0, 1, 2.5);

  /* Attributo Metallo */
  metallo = create_attribute("metallo");
  set_color(metallo, 0.71, 0.71, 0.71, 0.1,0.449);
  set_reflectivity(metallo, 0.2, 128);

  /* Attributo Plastica: Bianca, Nera, Rossa e Marrone */
  plastica_bianca = create_attribute("plastica_bianca");
  set_color(plastica_bianca, 0.98,0.98,0.98, 0.2,0.45);
  set_reflectivity(plastica_bianca, 0.05, 30);
  plastica_nera = create_attribute("plastica_nera");
  set_color(plastica_nera, 0.0,0.0,0.0, 0.2,0.45);
  set_reflectivity(plastica_nera, 0.05, 30);
  plastica_rossa = create_attribute("plastica_rossa");
  set_color(plastica_rossa, 0.9,0.0,0.0, 0.2,0.45);
  set_reflectivity(plastica_rossa, 0.05, 30);
  plastica_marrone = create_attribute("plastica_marrone");
  set_color(plastica_marrone, 0.643,0.337,0.114, 0.2,0.45);
  set_reflectivity(plastica_marrone, 0.05, 30);


  /* Load NURBS - Scena */
  pareti = read_obj("Pareti.obj", "pareti");
  set_attribute(pareti, intonaco);
  scena = create_list(pareti, "scena");

  soffitto = read_nurbs("parete_soffitto.db");
  set_bump_texture(soffitto, intonaco_chiaro, "textures/Deco.hr", 0,1,0,1, TRUE, 0.05,-1);
  scena = insert_in_list(scena, soffitto);

  pavimento = read_nurbs("parete_pavimento.db");
  set_domain_texture(pavimento, legno, "textures/Parquet.hr", 0,0.175,0,0.175, TRUE, 0.8,0.8);
  set_z_rotate(pavimento, 90, 1);
  scena = insert_in_list(scena, pavimento);

  tavolo_base = read_nurbs("tavolo_base.db");
  set_projection_texture(tavolo_base, intonaco, "textures/Marmo1.hr", 2, TRUE, 0.82,0.82);
  tavolo_piano = read_nurbs("tavolo_piano.db");
  set_scale(tavolo_piano, 0.4, 0.4, 0.4);
  set_x_rotate(tavolo_piano, 86, 1);
  set_projection_texture(tavolo_piano, intonaco, "textures/Marmo2.hr", 2, TRUE, 0.2,0.3);
  set_x_rotate(tavolo_piano, -86,1);
  set_scale(tavolo_piano, 2.5, 2.5, 2.5);
  scena = insert_in_list(scena, tavolo_base);
  scena = insert_in_list(scena, tavolo_piano);

  boccale_manico = read_nurbs("boccale_manico.db");
  set_attribute(boccale_manico, vetro);
  boccale = read_nurbs("boccale.db");
  set_attribute(boccale, vetro);
  scena = insert_in_list(scena, boccale);
  scena = insert_in_list(scena, boccale_manico);

  fgl1 = read_nurbs("foglio1.db");
  fgl2 = read_nurbs("foglio2.db");
  set_domain_texture(fgl1, carta, "textures/Foglio1.hr", 0,1,1,0, TRUE, 0.35,0.35);
  set_domain_texture(fgl2, carta, "textures/Foglio2.hr", 0,1,1,0, TRUE, 0.35,0.35);
  scena = insert_in_list(scena, fgl1);
  scena = insert_in_list(scena, fgl2);

  qdr1 = read_nurbs("quadro1.db");
  qdr2 = read_nurbs("quadro2.db");
  set_domain_texture(qdr1, intonaco, "textures/Quadro1.hr", 0,1,0,1, TRUE, 0.55,0.55);
  set_domain_texture(qdr2, intonaco, "textures/Quadro2.hr", 0,1,0,1, TRUE, 0.55,0.55);
  scena = insert_in_list(scena, qdr1);
  scena = insert_in_list(scena, qdr2);

  qdr1_c = read_nurbs("quadro1_cornice.db");
  qdr2_c = read_nurbs("quadro2_cornice.db");
  set_scale(qdr1_c, 10, 10, 10);
  set_z_rotate(qdr1_c, 68,1);
  set_x_rotate(qdr1_c, 88,1);
  set_wood_texture(qdr1_c, marrone_chiaro, marrone_scuro);
  set_x_rotate(qdr1_c, -88,1);
  set_z_rotate(qdr1_c, -68,1);
  set_scale(qdr1_c, 0.1, 0.1, 0.1);
  set_scale(qdr2_c, 30, 30, 30);
  set_z_rotate(qdr2_c, -85,1);
  set_x_rotate(qdr2_c, -65,1);
  set_wood_texture(qdr2_c, marrone_chiaro, marrone_scuro);
  set_x_rotate(qdr2_c, 65,1);
  set_z_rotate(qdr2_c, 85,1);
  set_scale(qdr2_c, 0.0333, 0.0333, 0.0333);
  scena = insert_in_list(scena, qdr1_c);
  scena = insert_in_list(scena, qdr2_c);

  lampadario = read_nurbs("lampadario.db");
  lamp_aggancio = read_nurbs("lampadario_aggancio.db");
  set_domain_texture(lampadario, ceramica, "textures/Immagine.hr", 0,0.5,0,0.5, TRUE, 0.45,0.45);
  set_attribute(lamp_aggancio, rame);
  lamp_asta = read_nurbs("lampadario_asta.db");
  set_attribute(lamp_asta, metallo);
  lamp_lampadina = read_nurbs("lampadario_lampadina.db");
  set_attribute(lamp_lampadina, vetro);
  scena = insert_in_list(scena, lampadario);
  scena = insert_in_list(scena, lamp_aggancio);
  scena = insert_in_list(scena, lamp_asta);
  scena = insert_in_list(scena, lamp_lampadina);

  sedia_seduta = read_nurbs("sedia_seduta.db");
  set_scale(sedia_seduta, 0.42184, 0.43402, 1);
  set_translate(sedia_seduta, 0.12, -0.53, 0.0);
  set_projection_texture(sedia_seduta, intonaco, "textures/Paglia.hr", 2, TRUE, 0.45,0.45);
  set_translate(sedia_seduta, -0.12, 0.53, -0.0);
  set_scale(sedia_seduta, 2.37052, 2.304, 1);
  sedia = read_obj("Sedia.obj", "sedia");
  set_domain_texture(sedia, legno, "textures/Legno1.hr", 0,1,0,0.5, TRUE, 0.35,0.35);
  scena = insert_in_list(scena, sedia_seduta);
  scena = insert_in_list(scena, sedia);

  mensola = read_nurbs("mensola.db");
  set_domain_texture(mensola, intonaco, "textures/Mattone.hr", 0,0.6,0,0.6, TRUE, 0.35,0.35);
  scena = insert_in_list(scena, mensola);

  specchio_superficie = read_nurbs("specchio.db");
  specchio_cornice = read_nurbs("specchio_cornice.db");
  set_attribute(specchio_superficie,specchio);
  set_domain_texture(specchio_cornice, ceramica, "textures/Mosaico.hr", 0,1,0,0.08, TRUE, 0.35,0.35);
  scena = insert_in_list(scena, specchio_superficie);
  scena = insert_in_list(scena, specchio_cornice);

  lbr1_copertina = read_nurbs("libro1_copertina.db");
  set_domain_texture(lbr1_copertina, carta, "textures/Copertina1.hr", 0,1,0,1, FALSE, 0.35,0.35);
  lbr1_costa = read_nurbs("libro1_costa.db");
  set_domain_texture(lbr1_costa, carta, "textures/Costa1.hr", 1,0,1,0, FALSE, 0.35,0.35);
  lbr1_pagine = read_nurbs("libro1_pagine.db");
  set_y_rotate(lbr1_pagine, -90, 1);
  set_scale(lbr1_pagine, 1, 0.5, 0.5);
  set_projection_texture(lbr1_pagine, carta, "textures/Pagine.hr", 0, TRUE, 0.4,0.35);
  set_scale(lbr1_pagine, 1, 2, 2);
  set_y_rotate(lbr1_pagine, 90, 1);
  scena = insert_in_list(scena, lbr1_copertina);
  scena = insert_in_list(scena, lbr1_costa);
  scena = insert_in_list(scena, lbr1_pagine);

  lbr2_copertina = read_nurbs("libro2_copertina.db");
  set_domain_texture(lbr2_copertina, carta, "textures/Copertina2.hr", 0,1,0,1, FALSE, 0.35,0.35);
  lbr2_costa = read_nurbs("libro2_costa.db");
  set_domain_texture(lbr2_costa, carta, "textures/Costa2.hr", 0,1,0,1, FALSE, 0.35,0.35);
  lbr2_pagine = read_nurbs("libro2_pagine.db");
  set_y_rotate(lbr2_pagine, -90, 1);
  set_scale(lbr2_pagine, 1, 1, 0.5);
  set_projection_texture(lbr2_pagine, carta, "textures/Pagine.hr", 0, TRUE, 0.3,0.25);
  set_scale(lbr2_pagine, 1, 1, 2);
  set_y_rotate(lbr2_pagine, 90, 1);
  lbr2_retro = read_nurbs("libro2_retro.db");
  set_domain_texture(lbr2_retro, carta, "textures/Retro2.hr", 0,1,0,1, FALSE, 0.6,0.5);
  scena = insert_in_list(scena, lbr2_copertina);
  scena = insert_in_list(scena, lbr2_costa);
  scena = insert_in_list(scena, lbr2_pagine);
  scena = insert_in_list(scena, lbr2_retro);

  lbr3_copertina = read_nurbs("libro3_copertina.db");
  set_domain_texture(lbr3_copertina, carta, "textures/Copertina3.hr", 0,1,0,1, FALSE, 0.35,0.35);
  lbr3_costa = read_nurbs("libro3_costa.db");
  set_domain_texture(lbr3_costa, carta, "textures/Costa3.hr", 1,0,1,0, FALSE, 0.35,0.35);
  lbr3_pagine = read_nurbs("libro3_pagine.db");
  set_y_rotate(lbr3_pagine, -90, 1);
  set_z_rotate(lbr3_pagine, -24.619997714, 1);
  set_scale(lbr3_pagine, 1, 1, 0.5);
  set_projection_texture(lbr3_pagine, carta, "textures/Pagine.hr", 0, TRUE, 0.4,0.35);
  set_scale(lbr3_pagine, 1, 1, 2);
  set_z_rotate(lbr3_pagine, 24.619997714, 1);
  set_y_rotate(lbr3_pagine, 90, 1);
  lbr3_retro = read_nurbs("libro3_retro.db");
  set_domain_texture(lbr3_retro, carta, "textures/Retro3.hr", 1,0,0,1, FALSE, 0.6,0.5);
  scena = insert_in_list(scena, lbr3_copertina);
  scena = insert_in_list(scena, lbr3_costa);
  scena = insert_in_list(scena, lbr3_pagine);
  scena = insert_in_list(scena, lbr3_retro);

  /* Load NURBS - Red Baron */
  parti_bianche = read_obj("PartiBianche.obj", "parti_bianche");
  set_attribute(parti_bianche, plastica_bianca);
  fokker = create_list(parti_bianche, "fokker");

  parti_nere = read_obj("PartiNere.obj", "parti_nere");
  set_attribute(parti_nere, plastica_nera);
  fokker = insert_in_list(fokker, parti_nere);

  parti_rosse = read_obj("PartiRosse.obj", "parti_rosse");
  set_attribute(parti_rosse, plastica_rossa);
  fokker = insert_in_list(fokker, parti_rosse);

  parti_legno = read_obj("PartiLegno.obj", "parti_legno");
  elica = read_nurbs("elica.db");
  set_domain_texture(parti_legno, legno, "textures/Legno2.hr", 0,0.2,0,0.2, TRUE, 0.35,0.35);
  set_domain_texture(elica, legno, "textures/Legno2.hr", 0,0.2,0,0.2, TRUE, 0.35,0.35);
  fokker = insert_in_list(fokker, parti_legno);
  fokker = insert_in_list(fokker, elica);

  parti_metalliche = read_obj("PartiMetalliche.obj", "parti_metalliche");
  set_attribute(parti_metalliche, metallo);
  fokker = insert_in_list(fokker, parti_metalliche);

  bordo_pilota = read_nurbs("bordo_pilota.db");
  set_attribute(bordo_pilota, plastica_marrone);
  fokker = insert_in_list(fokker, bordo_pilota);

  ala_I = read_nurbs("ala_I.db");
  set_attribute(ala_I, plastica_rossa);
  set_scale(ala_I, 1.15, 7, 1);
  set_translate(ala_I, -0.023, -10.55703, 0);
  set_projection_texture(ala_I, plastica_rossa, "textures/CroceD.hr", 2, FALSE, 0.2,0.45);
  set_translate(ala_I, 0.023, 10.55703, 0);
  set_scale(ala_I, 0.869565, 0.142857, 1);
  fokker = insert_in_list(fokker, ala_I);

  fusoliera_posteriore = read_nurbs("fusoliera_posteriore.db");
  set_attribute(fusoliera_posteriore, plastica_rossa);
  set_scale(fusoliera_posteriore, 9, 9, 11);
  set_translate(fusoliera_posteriore, -0.00026, -6.5, -2);
  set_projection_texture(fusoliera_posteriore, plastica_rossa, "textures/CroceS.hr", 0, FALSE, 0.2,0.45);
  set_translate(fusoliera_posteriore, 0.00026, 6.5, 2);
  set_scale(fusoliera_posteriore, 0.111111, 0.111111, 0.090909);
  fokker = insert_in_list(fokker, fusoliera_posteriore);

  timone = read_nurbs("timone.db");
  set_attribute(timone, plastica_bianca);
  set_scale(timone, 7, 7, 7);
  set_translate(timone, -0.00252, 0.7, -2.72);
  set_projection_texture(timone, plastica_bianca, "textures/CroceS.hr", 0, FALSE, 0.2,0.45);
  set_translate(timone, 0.00252, -0.7, 2.72);
  set_scale(timone, 0.142857, 0.142857, 0.142857);
  fokker = insert_in_list(fokker, timone);

  set_scale(fokker, 2.25, 2.25, 2.25);
  set_translate(fokker, -0.55864, -3.3525, 1.03284);  /* Carrello in (0,0,0) */
  set_x_rotate(fokker, 21, 1);
  set_z_rotate(fokker, 49, 1);
  set_translate(fokker, 30, -1.3, 17.51716);
  scena = insert_in_list(scena, fokker);

#ifndef ANIMAZIONI

  /* Rese Singole */
  save_scene("redbaron.md", "scena_redbaron", scena);

#else

  /* Animazioni */
  for (anim = 1; anim <= 100; anim++) {

    /*--- Zona Mensola ---*/
    /* sparizione Barone Rosso */
    if (anim == 1) {
      set_translate(fokker, -30.86891, 2.62563, -18.68108);      /* Baricentro (0,0,0) */
      set_scale(fokker, 0.000976562, 0.000976562, 0.000976562);  /* Scala: 1/1024 */
      set_translate(fokker, 30.86891, -2.62563, 18.68108);
      set_translate(fokker, 0.0, 0.0, step_apparizione[anim-1]); /* Allineamento sulla mensola */
      set_warn_light(0.0,0.0,22.44, 0.0,0.0,-1.0, 1,1,0.7, 0.0000001,35,0.3,45, 0,0,0,0,0, lampadina);
    }

    /* incremento luminoso e apparizione del Barone */
    if (1 < anim && anim < 10) {

      if (anim == 2) {
        fenice = create_point_light(30.86891,-2.62563,17.65,
                 step_fenice[anim-2][R],step_fenice[anim-2][G],step_fenice[anim-2][B],
                 0.2, 800, "fenice");
      } else {
        set_point_light(30.86891,-2.62563,17.65,
                 step_fenice[anim-2][R],step_fenice[anim-2][G],step_fenice[anim-2][B],
                 (5*anim),800, fenice);
      }

      set_translate(fokker, 0.0, 0.0, -step_apparizione[anim-2]);
      set_translate(fokker, -30.86891, 2.62563, -18.68108);
      set_scale(fokker, 2, 2, 2);                              /* Scala: 2^8 = 1024 */
      set_translate(fokker, 30.86891, -2.62563, 18.68108);
      set_translate(fokker, 0.0, 0.0, step_apparizione[anim-1]);
    }

    /* decremento luminoso e inizio movimento del Barone */
    if (10 <= anim && anim <= 20) {

      if (anim <= 11) {
        set_translate(fokker, 0.0, 0.0, -step_apparizione[anim-2]);
        set_translate(fokker, -30.86891, 2.62563, -18.68108);
        set_scale(fokker, 2, 2, 2);                              /* Scala: 2^2 = 16 */
        set_translate(fokker, 30.86891, -2.62563, 18.68108);
        if (anim != 11 ) set_translate(fokker, 0.0, 0.0, step_apparizione[anim-1]);
      }

      if (anim >= 15) {
        set_point_light(35.0,35.0,40.0, 0,0,0, 0.0000001,200, fenice);
      } else {
        set_point_light(30.86891,-2.62563,17.65,
                        step_fenice[7][R],step_fenice[7][G],step_fenice[7][B],
                        (40 - (5*(anim%9))),400, fenice);
      }

      if (anim >= 16) { /* rotazione elica */

        set_translate(elica, -30, 1.3, -17.51716);
        set_z_rotate(elica, -49, 1);
        set_x_rotate(elica, -21, 1);
        set_translate(elica, 0.55864, 3.3525, -1.34784); /* z = 1.03284 + 0.315 */
        set_y_rotate(elica, -2.5 * (anim%15), 1);
        set_translate(elica, -0.55864, -3.3525, 1.34784);
        set_x_rotate(elica, 21, 1);
        set_z_rotate(elica, 49, 1);
        set_translate(elica, 30, -1.3, 17.51716);
      }
    }

    /* decollo */
    if (20 < anim && anim <= 29) {

      /* coda alzata e primo avanzamento */
      if (24 <= anim && anim <= 29) {
        set_translate(fokker, -30, 1.3, -17.51716);
        set_z_rotate(fokker, -49, 1);
        set_translate(fokker, 0.0, -step_avanzamento, 0.0);
        set_x_rotate(fokker, -3, 1);
        step_avanzamento = step_avanzamento + 0.2 * (anim%23);
        set_translate(fokker, 0.0, step_avanzamento, 0.0);
        set_z_rotate(fokker, 49, 1);
        set_translate(fokker, 30, -1.3, 17.51716);
      }

      /* rotazione elica */
      set_translate(elica, -30, 1.3, -17.51716);
      set_z_rotate(elica, -49, 1);
      if (24 <= anim && anim <= 29) {
        set_translate(elica, 0.0, -step_avanzamento, 0.0);
        set_x_rotate(elica, -21 + (3*anim%23), 1);
      } else set_x_rotate(elica, -21, 1);
      set_translate(elica, 0.55864, 3.3525, -1.34784);

      if (anim <= 25) set_y_rotate(elica, -2.5*(anim%15), 1);
      else set_y_rotate(elica, -40-(2.5*(anim%25)), 1);

      set_translate(elica, -0.55864, -3.3525, 1.34784);
      if (24 <= anim && anim <= 29) {
        set_x_rotate(elica, 21 - (3*anim%23), 1);
        set_translate(elica, 0.0, step_avanzamento, 0.0);
      } else set_x_rotate(elica, 21, 1);
      set_z_rotate(elica, 49, 1);
      set_translate(elica, 30, -1.3, 17.51716);

    } else if (30 <= anim && anim <= 40) {

      /* avanzamento costante */
      set_translate(fokker, -30, 1.3, -17.51716);
      set_z_rotate(fokker, -49, 1);
      set_translate(fokker, 0.0, -step_avanzamento, 0.0);
      step_avanzamento = step_avanzamento + 1.6;
      set_translate(fokker, 0.0, step_avanzamento, 0.0);
      set_z_rotate(fokker, 49, 1);
      set_translate(fokker, 30, -1.3, 17.51716);

      set_translate(elica, -30, 1.3, -17.51716);
      set_z_rotate(elica, -49, 1);
      set_translate(elica, 0.0, -step_avanzamento, 0.0);
      set_x_rotate(elica, -3, 1);
      set_translate(elica, 0.55864, 3.3525, -1.34784);
      set_y_rotate(elica, -52, 1);  /* rotazione elica */
      set_translate(elica, -0.55864, -3.3525, 1.34784);
      set_x_rotate(elica, 3, 1);
      set_translate(elica, 0.0, step_avanzamento, 0.0);
      set_z_rotate(elica, 49, 1);
      set_translate(elica, 30, -1.3, 17.51716);

    }

    /*--- Movimento Camera Nella Scena e Atterraggio sul tavolo ---*/
    if ( 40 < anim && anim <= 100) {

      if (anim == 41) {
        set_translate(fokker, -30, 1.3, -17.51716);  /* nuova posizione del Barone */
        set_z_rotate(fokker, -49, 1);
        set_translate(fokker, 0.0, -step_avanzamento, 0.0);
        set_z_rotate(fokker, 140, 1);
        set_translate(fokker, 18, 23, 10.44216);
        step_avanzamento = 0;
      }

      if (54 < anim && anim <= 88) {

        /* riduzione luci */
        if (60 < anim && anim <= 65) {
          set_point_light(5.0,-30.0,26.0, 1,1,0.9, (1.2 - (0.1*(anim%60))), 100, luce1);
          set_point_light(-33.0,-30.0,19.5, 1,1,0.9, (1.5 - (0.12*(anim%60))), 180, luce2);
        }

        /* accensione lampadario */
        if (anim == 82 || anim == 83) {
          set_warn_light(0.0,0.0,22.44, 0.0,0.0,-1.0, 1,1,0.7, 2.75*(anim%81),35,0.3,45, 0,0,0,0,0, lampadina);
        }

        /* avanzamento costante */
        set_translate(fokker, -18, -23, -10.44216);
        set_z_rotate(fokker, -140, 1);
        set_translate(fokker, 0.0, -step_avanzamento, 0.0);

        if (72 <= anim && anim <= 78) {  /* cala la coda */
          if (76 <= anim && anim <= 88) set_z_rotate(fokker, -2.4*(anim%76), 1);
            set_x_rotate(fokker, -2*(anim%72), 1);
            set_x_rotate(fokker, 2*(anim%71), 1);
          if (76 <= anim && anim <= 88) set_z_rotate(fokker, 2.4*(anim%76), 1);
        }
        if (85 <= anim && anim <= 88) {
          if (76 <= anim && anim <= 88) set_z_rotate(fokker, -2.4*(anim%76), 1);
            set_x_rotate(fokker, -1*(anim%85), 1);
            set_x_rotate(fokker, 1*(anim%84), 1);
          if (76 <= anim && anim <= 88) set_z_rotate(fokker, 2.4*(anim%76), 1);
        }
        if (76 <= anim && anim <= 88) {  /* rotazione finale */
          set_z_rotate(fokker, -2.4*(anim%76), 1);
          set_z_rotate(fokker, 2.4*(anim%75), 1);
        }

        step_avanzamento = step_avanzamento + (1.25 - (0.025*(anim%55)));
        set_translate(fokker, 0.0, step_avanzamento, 0.0);
        set_z_rotate(fokker, 140, 1);
        set_translate(fokker, 18, 23, 10.44216);


        set_translate(elica, -18, -23, -10.44216);
        set_z_rotate(elica, -140, 1);
        set_translate(elica, 0.0, -step_avanzamento, 0.0);
        if (76 <= anim && anim <= 88) set_z_rotate(elica, -2.4*(anim%75), 1);
        if (anim < 72) set_x_rotate(elica, -3, 1);
        else if (72 <= anim && anim <= 78) set_x_rotate(elica, -3 - (2*(anim%71)), 1);
        else if (78 < anim && anim < 85) set_x_rotate(elica, -17, 1);
        else if (85 <= anim && anim <= 88) set_x_rotate(elica, -17 - (1*(anim%84)), 1);
        set_translate(elica, 0.55864, 3.3525, -1.34784);
        if (anim < 78 ) set_y_rotate(elica, -52, 1);  /* rotazione elica */
        else set_y_rotate(elica, (-52 + 5*(anim%78)), 1);
        set_translate(elica, -0.55864, -3.3525, 1.34784);
        if (anim < 72) set_x_rotate(elica, 3, 1);
        else if (72 <= anim && anim <= 78) set_x_rotate(elica, 3 + (2*(anim%71)), 1);
        else if (78 < anim && anim < 85) set_x_rotate(elica, 17, 1);
        else if (85 <= anim && anim <= 88) set_x_rotate(elica, 17 + (1*(anim%84)), 1);
        if (76 <= anim && anim <= 88) set_z_rotate(elica, 2.4*(anim%75), 1);
        set_translate(elica, 0.0, step_avanzamento, 0.0);
        set_z_rotate(elica, 140, 1);
        set_translate(elica, 18, 23, 10.44216);
      }
    }

    save_scene_ani(100, "redbaron.ani","redbaron", "scena_redbaron", scena);
  }

#endif

  return(0);
}
